/**
 * \file lexer.c
 *
 * \brief Implémentation du lexer MT avec re2c
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020-2021
 */

/*

Copyright (C) 2020-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

Ce fichier fait partie de Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

---------------------------------------

This file is part of Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "driver.h"
#include "parser.h"


/**
  * \fn int MTScan(MTScanner* s)
  * \brief La fonction MTScan est le "lexer" qui recherche les "tokens" dans le texte fourni.
  *
  * \param s : pointeur vers la structure qui permet de suivre le parcours du texte par le lexer
  *
  * \return la référence du token identifié
  */
int MTScan(MTScanner* s)
{
start:
  s->startPos = s->curPos;

  /*!re2c

    re2c:define:YYCTYPE = int;
    re2c:define:YYCURSOR = s->curPos;
    re2c:define:YYLIMIT = s->endPos;
    re2c:define:YYMARKER = s->re2c;

    re2c:yyfill:enable = 0;
    re2c:eof = 0;

    space = [ \t\xFF\xA0]+;
    endOfLine = "\r\n" | "\n";
    number = [0-9];
    integer = (number)+;
    doubleString = ["] [^"\r\n]* ["];
    singleString = ['] [^'\r\n]* ['];
    substring = (doubleString) | (singleString);
    sub = 'SUB' [^"'()[\]]*;
    subt = 'SUBT' [^"'()[\]]*;
    loc = 'LOC';
    loci = 'LOCI'|'FIND';
    comment = "//" [^\r\n]*;
    value = 'VAL';
    string = 'STR';

    "("                   { return OPARENTHESIS; }
    ")"                   { return CPARENTHESIS; }
    "["                   { return OBRACKET; }
    "]"                   { return CBRACKET; }
    [,;]                  { return SEPARATOR; }

    "."                   { return POINT; }
    "="                   { return EQUAL; }
    "<>"|"!="             { return DIFF; }
    "+"                   { return PLUS; }
    "-"                   { return MINUS; }
    "*"                   { return MULTIPLY; }
    "/"                   { return DIVIDE; }
    "<"                   { return LT; }
    ">"                   { return GT; }
    "<="                  { return LTE; }
    ">="                  { return GTE; }

    'if'                  { s->line ++; return END; }
    'et'|'and'|"&"        { return AND; }
    'ou'|'or'|"|"         { return OR; }
    'non'|'not'|"!"       { return NOT; }

    integer               { return INTEGER; }
    substring             { return SUBSTRING; }
    space                 { goto start; }

    endOfLine             { s->line ++; s->lineStart = s->curPos; return EOL; }

    subt                  { return SUBT; }
    sub                   { return SUB; }
    loci                  { return LOCI; }
    loc                   { return LOC; }

    value                 { return VALUE; }
    string                { return STRING; }
    comment               { goto start; }

    *                     { goto start; }
    $                     { s->line ++; return END; }
  */
}
