/**
 * \file reader.c
 *
 * \brief Implémentation du lecteur de fichiers
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020
 */

/*

Copyright (C) 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

Ce fichier fait partie de Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

---------------------------------------

This file is part of Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "reader.h"


typedef struct linkedString linkedString;

struct linkedString
{
  int index;
  char *string;
  linkedString *next;
};

#define AddLinkedString(index, newString) ManageLinkedString(index, newString)
#define ReplaceLinkedString(index, newString) ManageLinkedString(index, newString)
#define GetLinkedString(index, newString) ManageLinkedString(index, NULL)

/**
  * \fn static char *ManageLinkedString(int index, char *newString)
  * \brief La fonction permet de stocker ou chercher une chaîne selon un index à partir de 1
  *
  * \param index : index de la chaîne (strictement positif)
  * \param newString : pointeur temporaire vers la nouvelle chaîne (elle sera dupliquée)
  *
  * \return pointeur sur la chaîne demandée ou un pointeur NULL dans le cas contraire
  */
static char *ManageLinkedString(int index, char *newString)
{
  static linkedString *ls0 = NULL;
  char *string = NULL;

  if (index > 0)
    {
      linkedString *ls = ls0, *lsOld = ls0;

      while ((ls != NULL) && (ls->index != index))
        {
          lsOld = ls;
          ls = ls->next;
        }

      if (ls == NULL)
        {
          ls = calloc(1, sizeof(linkedString));

          if (lsOld != NULL)
            {
              lsOld->next = ls;
            }
          else
          {
            ls0 = ls;
          }
        }

      if (ls != NULL)
        {
          ls->index = index;

          if (newString != NULL)
            {
              if (ls->string != NULL)
                {
                  free(ls->string);
                }

              ls->string = strdup(newString);
            }

          string = ls->string;
        }
    }

  return string;
}


/**
  * \fn const char *ReadLineInFile(int relativeIndex)
  * \brief La fonction retourne la ligne demandée du fichier test
  *
  * \param relativeIndex : index relatif de la ligne à lire par rapport à la ligne actuelle
  *                        (0 pour la ligne courante, 1 pour la suivante, -1 pour la précédente)
  *
  * \return pointeur sur la ligne demandée (n'est jamais NULL et ne doit pas être libéré)
  */
const char *ReadLineInFile(int relativeIndex)
{
  static int absoluteIndex = 3; /**< index absolu depuis le début du fichier, commence à 1 */
if (relativeIndex == 10)
{
  relativeIndex += absoluteIndex - 3;
}
  int index = absoluteIndex + relativeIndex;

  char *result = GetLinkedString(index, NULL);

  if (result == NULL)
    {
      FILE *file;

      file = fopen("./test.txt", "rb");
      if (file == NULL)
        {
          fprintf(stderr, "Impossible d'ouvrir le fichier texte\n");
          exit(-1);
        }

      fseek(file, 0, SEEK_END);
      size_t taille = ftell(file);
      fseek(file, 0, SEEK_SET);

      char buffer[taille + 1];
      buffer[0] = '\0';

      int count = 0;
      while (count < index)
        {
          count ++;

          if (fgets(buffer, sizeof(buffer), file) == NULL)
            {
              buffer[0] = '\0';
              break;
            }
        }

      fclose(file);

      if ((buffer[0]) && (buffer[strlen(buffer) - 1] == '\n'))
        {
          buffer[strlen(buffer) - 1] = '\0';
        }

      result = AddLinkedString(index, buffer);
    }

  return result ? result : "";
}

