/**
 * \file message.c
 *
 * \brief Implémentation des fonctions et macros pour la gestion des messages (version GTK)
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include "../../taupie.h"
#include "../message.h"


/**
 * \fn int MinDMessageBox(GtkWidget* hInstance, GtkWidget* windowParent, MinDMessage *message);
 * \brief Création et affichage d'un message d'information.
 *
 * S'il n'y a ni texte ni titre, la boîte de message n'est pas lancée, aucun message n'est affiché.
 *
 * \param hInstance : instance de l'application
 * \param windowParent : handle de la fenêtre parent
 * \param message : pointeur vers la structure du message à afficher
 *
 * \return 0 si tout s'est bien passé
 * \return -1 en cas de problème
 */
int MinDMessageBox(GtkWidget* hInstance, GtkWidget* windowParent, MinDMessage *message)
{
  int result = -1;

  if (message)
    {
      const gchar *text = message->text;
      const gchar *title = message->title;

      GtkWidget *window = gtk_message_dialog_new(GTK_WINDOW(NULL),
                                                 GTK_DIALOG_MODAL,
                                                 GTK_MESSAGE_INFO,
                                                 GTK_BUTTONS_OK,
                                                 TEXT("%s"), text ? text : TEXT(""));

      //si la fenêtre parent est utilisée, cela ne marche pas !
      /*GtkWidget *window = gtk_message_dialog_new(GTK_WINDOW(windowParent),
                                                   GTK_DIALOG_MODAL,
                                                   GTK_MESSAGE_INFO,
                                                   GTK_BUTTONS_OK,
                                                   TEXT("%s"), text ? text : TEXT(""));*/

      gtk_window_set_title(GTK_WINDOW(window), title ? title : TEXT(""));

#if GTK_MAJOR_VERSION < 4
      gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
#endif

      gtk_widget_show_all(window);
      result = gtk_dialog_run(GTK_DIALOG(window));
      gtk_widget_destroy(GTK_WIDGET(window));

    }

  return result;
}
