#ifndef DIALOG_INCLUDED
#define DIALOG_INCLUDED


/**
 * \file dialog.h
 *
 * \brief Déclaration des fonctions et macros pour la gestion des dialogues
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#ifdef UI_GTK
#include "../GTK/ui.h"
#endif

#ifdef UI_WIN32
#include "../Win32/ui.h"
#endif

#include "mind.h"
#include "variable.h"

#define ITEM_TYPE_MASK      0xF000
#define ITEM_GROUP_MASK     0xFF00
#define ITEM_INDEX_MASK     0x00FF

#define BUTTON_ID           0x1000
#define RADIO_ID            0x3000
#define STATIC_ID           0x5000
#define EDITBOX_ID          0x6000
#define CHECKBOX_ID         0x7000
#define DROPBOX_ID          0x8000


/**
 * \struct _DialogLine
 * \brief Structure de stockage des lignes de dialogue
 */
struct _DialogLine
{
  int position;                       /**< line number in file */
  struct _DialogLine *previous;       /**< pointeur vers la structure précédente */
};

typedef struct _DialogLine DialogLine;


/**
 * \struct _MinDBox
 * \brief Structure de stockage des dimensions d'une boîte
 */
struct _MinDBox
{
  short int x;                /**< position X */
  short int y;                /**< position Y */
  short int width;            /**< largeur */
  short int height;           /**< hauteur */
};

typedef struct _MinDBox MinDBox;


#define DLG   1 /**< DialogBox */
#define BTN   2 /**< ButtonSet */
#define RDO   3 /**< RadioGroup */
#define TXT   4 /**< StaticText */
#define EDT   5 /**< EditBox */
#define CHK   6 /**< CheckBox */
#define DRP   7 /**< DropBox */
#define ITM   8 /**< DBItem */

/**
 * \struct _MinDInput
 * \brief Structure de stockage d'un élément de dialogue
 */
struct _MinDDialog
{
  char type;                /**< type de l'élément (item) */
  HWND item;                  /**< handle de l'item dans l'UI */
  int itemID;                 /**< identifiant de l'item + sélection (radio/button) */
  char *title;              /**< titre ou texte à afficher */
  char *varName;            /**< nom de la variable de destination */
  MinDBox box;                /**< paramètres graphiques d'un élément de dialogue */
  char *defT;               /**< texte par défaut */
  short int defI;             /**< numéro par défaut (numéro de bouton, liste déroulante) */
  char option;              /**< 'C', 'R', 'H' ou 'K' pour Checked */
  struct _MinDDialog *next;   /**< pointeur vers l'entrée suivante */
};

typedef struct _MinDDialog MinDDialog;


void DisplayDialogBox(MinDState *mindstate, MinDDialog *dialog);
MinDDialog *CreateDialogBox(char type, MinDValue *title, MinDValue *varName, MinDBox box, MinDValue *def, char option);
int FreeDialogBox(MinDDialog *dialog);

int MinDDialogBox(HINSTANCE hInstance, HWND hwnd, MinDDialog *dialog);


#endif // DIALOG_INCLUDED
