#ifndef HEADER_DE72ABCCF25DFDB8
#define HEADER_DE72ABCCF25DFDB8

#ifdef MINDDRIVER_INCLUDED
#else
#define MINDDRIVER_INCLUDED


/**
 * \file driver.h
 *
 * \brief Déclarations et définitions pour l'analyseur des formules de champs
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020-2021
 */

/*

Copyright (C) 2020-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

Ce fichier fait partie de Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

---------------------------------------

This file is part of Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

*/


#ifdef UI_GTK
#include "../GTK/ui.h"
#endif

#ifdef UI_WIN32
#include "../Win32/ui.h"
#endif

#include "mind.h"
#include "variable.h"
#include "dialog.h"

#define Call(a) { if (a) { mindstate->state |= MINDSTATE_CALL; mindstate->s = a; }  else { mindstate->state = 0; } }


/**
 * \struct MinDScanner
 * \brief La structure permet de suivre le parcours du texte par le lexer
 */
struct MinDScanner
{
  const char *re2c;        /**< réservé au fonctionnement interne du lexer = YYMARKER */
  const char *startPos;    /**< récupère l'adresse de départ du token courant */
  const char *curPos;      /**< adresse du caractère courant lu par le lexer = YYCURSOR */
  const char *endPos;      /**< adresse du dernier caractère à lire (méthode EOF) = YYLIMIT */
  const char *lineStart;   /**< récupère l'adresse de départ de la ligne courante */
  int line;                  /**< numéro de la ligne en cours (indexée à 0) */
};

/**
 * \typedef MinDScanner
 * \brief Type défini à l'identique pour la structure MinDScanner
 */
typedef struct MinDScanner MinDScanner;

/**
 * \typedef MinDToken
 * \brief Type défini à l'identique que la structure _MinDValue.
 */
typedef MinDValue MinDToken;

int MinDDriver(MinDState *mindstate, const char *buffer);

int MinDScan(MinDScanner* s);

void MinDParse();
void *MinDParseAlloc();
void MinDParseFree();
void MinDParseTrace();
void MinDParseInit();
void MinDParseFinalize();
int MinDParseFallback();


#endif

#endif // header guard 

