#ifndef INPUT_INCLUDED
#define INPUT_INCLUDED

/**
 * \file input.h
 *
 * \brief Déclaration des fonctions et macros pour la gestion des dialogues simplifiés
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#ifdef UI_GTK
#include "../GTK/ui.h"
#endif

#ifdef UI_WIN32
#include "../Win32/ui.h"
#endif

#include "mind.h"
#include "variable.h"
#include "dialog.h"


/**
 * \struct _MinDInput
 * \brief Structure de stockage d'une demande interactive d'informations
 */
struct _MinDInput
{
  char *title;           /**< texte à afficher pour préciser la demande */
  HWND item;               /**< handle de l'item dans l'UI */
  int itemID;              /**< identifiant de l'item */
  char *varName;         /**< nom de la variable de destination */
  char *fields;          /**< nom de la variable de destination */
  struct _MinDInput *next; /**< pointeur vers l'entrée suivante */
};

typedef struct _MinDInput MinDInput;

void DisplayInputBox(MinDState *state, MinDInput *input);
MinDInput *CreateInputBox(MinDValue *value, MinDValue *varName, MinDValue *fields);
int FreeInputBox(MinDInput *input);

int MinDInputBox(HINSTANCE hInstance, HWND hwnd, MinDInput *input);


#endif

