/**
 * \file message.c
 *
 * \brief Implémentation des fonctions et macros pour la gestion des messages
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020
 */


/*

Copyright (C) 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#include "../taupie.h"
#include "message.h"


/**
  * \fn MinDMessage *CreateMessageBox(MinDValue *text, MinDValue *title)
  * \brief La fonction crée la structure d'un message d'information.
  *
  * \param text : texte du message à afficher sous forme de MinDValue.
  * \param title : titre du message à afficher sous forme de MinDValue..
  *
  * \return pointeur vers la structure du message (à libérer).
  */
MinDMessage *CreateMessageBox(MinDValue *text, MinDValue *title)
{
  MinDMessage *result = calloc(1, sizeof(MinDMessage));

  if (result)
    {
      result->text = GetItemText(text);
      result->title = GetItemText(title);
    }
  else
    {
      PushError(MESSAGE_INFO_MEMORYERROR);
    }

  return result;
}


/**
  * \fn void DisplayMessageBox(MinDState *mindstate, MinDMessage *message)
  * \brief La fonction affiche un message d'information.
  *
  * \param mindstate : pointeur vers la structure de résultat du parser
  * \param message : pointeur vers la structure stockant le message
  */
void DisplayMessageBox(MinDState *mindstate, MinDMessage *message)
{
  mindstate->state = MINDSTATE_MESSAGE;

  MinDMessageBox(mindstate->hInstance, mindstate->hwnd, message);

  FreeMessageBox(message);

  return;
}


/**
  * \fn int FreeMessageBox(MinDMessage *message)
  * \brief La fonction libère une structure de demande d'informations.
  *
  * \param message : pointeur vers la structure stockant le message d'information.
  *
  * \return le nombre d'éléments libérés.
  */
int FreeMessageBox(MinDMessage *message)
{
  int result = 0;

  if (message)
    {
      result ++;

      TaupieFree(message->text);
      TaupieFree(message->title);

      free(message);
    }

  return result;
}
