#ifndef MIND_INCLUDED
#define MIND_INCLUDED


/**
 * \file mind.h
 *
 * \brief Déclaration de quelques structures génériques de MinD
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2020
 */


/*

Copyright (C) 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, included in Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, inclus dans Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#ifdef UI_GTK
#include "../GTK/ui.h"
#endif

#ifdef UI_WIN32
#include "../Win32/ui.h"
#endif


#define MINDSTATE_NUL            0x0000  /**< pas d'information du parser */
#define MINDSTATE_INPUT          0x0001  /**< le parser a demandé quelque chose */
#define MINDSTATE_MESSAGE        0x0002  /**< le parser a affiché quelque chose */
#define MINDSTATE_ASSIGN         0x0003  /**< le parseur a affecté une valeur à une variable */
#define MINDSTATE_DELETE         0x0004  /**< le parseur a supprimé une variable */
#define MINDSTATE_DIALOG         0x0005  /**< le parseur a affiché une boîte de dialogue */

#define MINDSTATE_SYNTAXERROR    0x0020  /**< erreur de syntaxe du parser */
#define MINDSTATE_PARSEFAILURE   0x0040  /**< échec de l'analyse du parser */
#define MINDSTATE_SUCCESS        0x0080  /**< succès de l'analyse globale */

#define MINDSTATE_STOP           0x0100  /**< le parseur demande de sortir de tous les scripts */
#define MINDSTATE_ALLSTOP        0x0200  /**< le parseur demande de sortir du script courant */
#define MINDSTATE_CALL           0x0400  /**< le parseur appelle un script */
#define MINDSTATE_LOOP           0x0800  /**< le parseur demande une boucle */


/**
 * \struct _MinDValue
 * \brief Structure de stockage d'une valeur.
 *
 * Cette structure permet de stocker une valeur, quel que soit
 * le type de cette valeur, voire sous plusieurs formes.
 */
struct _MinDValue
{
  char type;     /**< type de valeur : L, S, N ou nul (pas de valeur) */
  char l;        /**< valeur sous forme de booléen */
  char *s;     /**< valeur sous forme de chaîne */
  double n;      /**< valeur sous forme numérique */
  char decimals; /**< nombre de décimales significatives de la forme numérique */
};


/**
 * \typedef MinDValue
 * \brief Type défini pour la structure MinDValue.
 */
typedef struct _MinDValue MinDValue;


/**
 * \struct _MinDState
 * \brief La structure reçoit les résultats et messages du parser.
 */
struct _MinDState
{
  HINSTANCE hInstance; /**< instance de l'application */
  HWND hwnd;           /**< handle de la fenêtre parent */
  int state;           /**< état courant du parser */
  int result;          /**< résultat éventuel du parser */
  char *s;           /**< chaîne éventuelle passée par le parser (à libérer) */
};


/**
 * \typedef MinDState
 * \brief Type défini pour la structure MinDState.
 */
typedef struct _MinDState MinDState;


/**
 * \struct _MinDNumber
 * \brief Structure de stockage d'un nombre
 */
struct _MinDNumber
{
  double n;     /**< valeur numérique */
  int decimals; /**< nombre de décimales significatives */
};

/**
 * \typedef MinDNumber
 * \brief Type défini pour la structure MinDNumber.
 */
typedef struct _MinDNumber MinDNumber;


char *GetItemText(MinDValue *value);
MinDNumber GetItemNumber(MinDValue *value);
int GetItemInteger(MinDValue *value);


char *fSubstr(MinDValue *value, MinDValue *position, MinDValue *maxLength);
MinDNumber fValue(const char *text, double decimals);
char *fString(MinDValue *value, MinDValue *length, MinDValue *format);


#endif // MIND_INCLUDED
