#ifndef VARIABLE_INCLUDED
#define VARIABLE_INCLUDED

/**
 * \file variable.h
 *
 * \brief Déclaration des fonctions et macros pour la gestion des variables
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2020
 */


/*

Copyright (C) 2017, 2020 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file is part of MinD, itself part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------------

Ce fichier fait partie de MinD, lui-même partie de Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.

*/


#ifdef UI_GTK
#include "../GTK/ui.h"
#endif

#ifdef UI_WIN32
#include "../Win32/ui.h"
#endif

#include "mind.h"


/**
 * \struct _MinDVar
 * \brief Structure de stockage d'une variable
 *
 * Cette structure permet de stocker les différents éléments d'une variable,
 * comme son nom, son type, sa valeur et de chaîner cette variable avec les autres
 * variables déclarées.
 */
struct _MinDVar
{
  char *name;                 /**< nom de la variable */
  MinDValue value;              /**< valeur de la variable, à choisir dans la structure selon le type */
  struct _MinDVar *previous;    /**< pointeur vers la variable précédente */
  struct _MinDVar *next;        /**< pointeur vers la variable suivante */
};

typedef struct _MinDVar MinDVar;

MinDVar *MinDVariable_Fill(MinDVar **mindVarPtr, const char *name, const MinDValue value);
MinDVar *MinDVariable_Link(MinDVar **mindVarPtr, MinDVar *var);
MinDVar *MinDVariable_Retrieve(MinDVar **mindVarPtr, char *name);
int MinDVariable_FreeAll(MinDVar **mindVarPtr, FILE *fileOut);
int MinDVariable_Clean(MinDVar **mindVarPtr);
char *MinDVariable_ReplaceVariableInString(MinDVar **mindVarPtr, char *text);
int MinDVariable_Delete(MinDVar **mindVarPtr, char *name);

MinDVar *CreateVariable(MinDValue *name, MinDValue *value);
MinDVar *AssignVariable(MinDState *state, MinDVar *var);
int FreeVariable(MinDVar *var);
int DeleteVariable(MinDState *state, char *name);

MinDValue *GetVarValue(MinDVar **mindVarPtr, MinDValue *value);

char *GetVarText(MinDVar **mindVarPtr, const char *varName);
char GetVarChar(MinDVar **mindVarPtr, const char *varName);
int GetVarInteger(MinDVar **mindVarPtr, const char *varName);



#endif

