/**
 * \file main.c
 *
 * \brief Implémentation de la procédure de fenêtre et de l'interface (version Win32)
 *
 * \author Denis MARTIN
 * \copyright GNU General Public License 3+
 *
 * \date 2017-2021
 *
 */


/*

Copyright (C) 2017-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

This file was part of MinOTAuR, wich is part of Taupie since 2020.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

--------------------------------------------------------------------------

Ce fichier faisait partie de MinOTAuR, qui est intégré à Taupie depuis 2020.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

*/


/**
 * \mainpage Taupie

### C'est quoi Taupie ?

L'objectif de Taupie est de fournir un petit outil léger, facile à mettre en oeuvre, permettant d'analyser
des données de diverses origines, sans qu'il soit nécessaire d'avoir des connaissances avancées en matière de
traitement de données.
Les projets créés avec Taupie sont des bases SQLite en UTF-8, soit un format libre et très répandu, sans gestion
client/serveur ni partage de droits, adapté aux performances d'un PC ordinaire, et facile à reprendre
avec d'autres outils.

* Taupie ne vise en aucun cas à concurrencer des outils spécifiques puissants réservés à des spécialistes de haut niveau
* maîtrisant des langages dédiés à l'analyse de données volumineuses, hétérogènes, permettant de produire des études
* statistiques avancées. L'utilisation de Taupie relève d'une démarche plus simple et sans visée scientifique,
* destinée à un public de non spécialistes ayant à auditer des données ciblées dans un temps et avec des moyens limités.

### Version actuelle de Taupie

#### Taupie 0.15.4 :

- Cette version de Taupie est limitée en possibilités, il s'agit essentiellement d'une interface
graphique pour SQLite3, fonctionnant avec l'API native Win32 sous Windows 64 bits.
- Elle intègre une fonctionnalité d'importation/exportation au format texte délimité et la possibilité de requêter librement
sur la base. Il est possible d'ouvrir n'importe quelle base SQLite3 encodée en UTF-8 avec Taupie, cela modifie toutefois
la base par l'ajout d'une table historisée des commandes (taupie_log).
- Son numéro de version majeur passera à 1 lorsque Taupie intégrera toutes les fonctionnalités prévues. Son numéro
de version mineur vaut 15 parce que Taupie intègre actuellement environ 15% des fonctionnalités prévues pour la version
majeure 1. Le dernier numéro (Build) est incrémenté à chaque version publiée.

### Comment utiliser Taupie ?

- Dans sa version Windows 64bits avec l'API native Win32, Taupie se présente sous la forme d'un exécutable
unique et ne nécessite aucune installation particulière.
- Toutefois, il utilise l'interface UTF-8 disponible sous Windows 10 (version 1903 mai 2019) et peut donc
présenter des anomalies d'affichage sous des versions antérieures de Windows.

### Où télécharger Taupie ?

Code source : <a href="../../download/TaupieSrc.zip"> TaupieSrc.zip </a>

Exécutable compilé : <a href="../../download/TaupieExe.zip"> TaupieExe.zip </a>

* L'exécutable contenu dans l'archive TaupieExe.zip correspond à la cible WinRelease64
* du projet Code::Blocks TaupieWin64 présent dans l'archive TaupieSrc.zip (code source).
* Il a été compilé statiquement dans Code::Blocks 20.03 avec gcc version 10.3.0 (Rev2, MSYS2, UCRT64)
* et liés statiquement avec la bibliothèque SQLite (version 3.35.5).

### Mentions légales

Copyright (C) 2020-2021 Denis MARTIN <dmartin@taupie.net>

---------------------------------------

Ce fichier fait partie de Taupie.

Taupie est un logiciel libre : il peut être redistribué et/ou modifié
selon les termes de la Licence Publique Générale GNU telle que publiée par
la Free Software Foundation, soit la version 3 de la licence ou toute
version ultérieure.

Taupie est distribué dans l'espoir qu'il sera utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de
VALEUR MARCHANDE ou d'ADEQUATION AVEC UN BUT PARTICULIER. Voir la
Licence Publique Générale GNU pour plus de détails.

Une copie de la Licence Publique Générale GNU devrait être fournie
avec Taupie. Si ce n'est pas le cas, voir <http://www.gnu.org/licenses/gpl.html>.

---------------------------------------

This file is part of Taupie.

Taupie is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License,
or any later version.

Taupie is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Taupie.  If not, see <http://www.gnu.org/licenses/gpl.html>.

*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>

#include <windows.h>
#include <commctrl.h>

#include <tchar.h>
#include <stdio.h>

#include <sqlite3.h>

#include "resource.h"
#include "../version.h"

#include "../taupie.h"
#include "../MinD/script.h"

#include "../MinOTAuR/Win32/minotaur.h"



/**
  * \fn int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, char *lpCmdLine, int nShowCmd)
  * \brief Procédure principale - point d'entrée du programme
  *
  * \param hInstance : instance du programme
  * \param hPrevInstance : toujours NULL
  * \param lpCmdLine : paramètres de la ligne de commande
  * \param nShowCmd : affichage demandé
  *
  * \return zéro si le programme se termine normalement
  * \return une valeur non nulle dans le cas contraire
  */
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, char *lpCmdLine, int nShowCmd)
{
  setlocale(LC_NUMERIC, "French_France.utf8");

  sqlite3_initialize();

  // évite le plantage en cas d'utilisation de GetOpenFileName() sur le Bureau !!!
  OleInitialize(NULL);

  // initialisation permettant l'affichage des contrôles avancés
  INITCOMMONCONTROLSEX initCC;
  initCC.dwSize = sizeof(INITCOMMONCONTROLSEX);
  initCC.dwICC = ICC_DATE_CLASSES | ICC_PROGRESS_CLASS | ICC_WIN95_CLASSES;
  InitCommonControlsEx(&initCC);

  char *line = lpCmdLine;

  WNDCLASSEX winClass;

  winClass.hInstance = hInstance;
  winClass.lpszClassName = "TaupieMainWindow";
  winClass.lpfnWndProc = Main_Procedure;
  winClass.style = CS_HREDRAW | CS_VREDRAW;
  winClass.cbSize = sizeof(WNDCLASSEX);

  winClass.hIcon = LoadIcon(hInstance, "IconTaupie_x8");
  winClass.hIconSm = LoadIcon(hInstance, "IconTaupie_x8");
  winClass.hCursor = LoadCursor(NULL, IDC_ARROW);
  winClass.lpszMenuName = "MainMenu";
  winClass.cbClsExtra = 0;
  winClass.cbWndExtra = 0;
  winClass.hbrBackground = (HBRUSH) COLOR_BACKGROUND;

  if (!RegisterClassEx(&winClass))
    {
      return 0;
    }

  HWND hwnd = CreateWindowEx(
           0,
           "TaupieMainWindow",
           "Taupie",

           // fenêtre sans barre de titre, donc non modifiable et non déplaçable
           //WS_POPUPWINDOW|WS_MAXIMIZE,

           // ou fenêtre globale avec barre de titre, déplaçable
           //WS_OVERLAPPED|WS_MINIMIZEBOX|WS_SYSMENU|WS_MAXIMIZE,

           // ou fenêtre modifiable et déplaçable avec barre de titre
           WS_OVERLAPPEDWINDOW | WS_MAXIMIZE | WS_CLIPSIBLINGS,

           CW_USEDEFAULT,
           CW_USEDEFAULT,
           CW_USEDEFAULT,
           CW_USEDEFAULT,
           HWND_DESKTOP,                // fenêtre parent = bureau
           NULL,
           hInstance,
           NULL
         );

  ShowWindow(hwnd, SW_SHOWMAXIMIZED);

  PostMessage(hwnd, WM_COMMAND, MAKEWPARAM(InitScript, 0) ,(LPARAM)line);

  MSG msg;

  while (GetMessage(&msg, NULL, 0, 0))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  int result = msg.wParam;

  return result;
}


